==> fly-qdm_2.9.6.tar.xz: fly-qdm/dm_greet_modern/themewidget.cpp <==
#include "themewidget.h"

#include "exceptions.h"
#include "themewidgetcreator.h"
#include "utils.h"
#include "widgets/loginwidget.h"
#include "widgets/commonwidget.h"
#include "managers/virtualkeyboardmanager.h"
#include "managers/keyboardmanager.h"
#include "managers/shutdownmanager.h"
#include "managers/sessionsmanager.h"
#include "managers/connectionmanager.h"
#include "managers/consoleauthmanager.h"

#include <QGridLayout>
#include <QDir>
#include <QMessageBox>
#include <QAction>
#include <QTranslator>
#include <QGraphicsBlurEffect>


ThemeWidget::ThemeWidget(QWidget *parent) :
    BaseThemeWidget(parent),
    m_themeDescription("") // TODO
{
    m_loginFailedTimer.setInterval(2000); // TODO
}

ThemeWidget::~ThemeWidget()
{
}

void ThemeWidget::load(const ThemeDescription& themeDescription, GreeterManagers* managers, const EnvironmentData& environment)
{
    m_environment = environment;
    m_themeDescription = themeDescription;
    insertManagers(managers);
    loadTranslations();

    m_sessionsManager = findManager<SessionsManager*>("sessions");

    const auto themePath = m_themeDescription.dirPath();

    m_commonWidget = new CommonWidget();
    m_commonWidget->load(ThemeWidgetCreator::create(themePath + "/common_widget.ui"));
    m_commonWidget->setAvailableSessions(m_sessionsManager->availableSessions());

    m_loginWidget = new LoginWidget();
    const auto sourceLoginWidget = ThemeWidgetCreator::create(themePath + "/central_widget.ui");
    m_loginWidget->load(sourceLoginWidget, m_themeDescription);

    m_backgroundWidget = new QWidget(this);
    m_backgroundWidget->setObjectName("backgroundWidget");
    const auto blurEffect = new QGraphicsBlurEffect();
    blurEffect->setBlurRadius(20.0); // TODO
    m_backgroundWidget->setGraphicsEffect(blurEffect);
    setBackgroundImage(themePath + "/background.png");

    const auto layout = new QGridLayout();
    layout->setMargin(0);
    layout->addWidget(m_commonWidget);
    setLayout(layout);

    const auto centralGridLayout = findWidget<QGridLayout*>("centralGridLayout");
    centralGridLayout->addWidget(m_loginWidget);

    const auto dateTimeWidget = findWidget<QWidget*>("dateTimeWidget");
    m_dateTimeWidget.load(dateTimeWidget);
    m_dateTimeWidget.start();

    m_usersManager = findManager<UsersManager*>("users");

    m_keyboardManager = findManager<KeyboardManager*>("keyboard");
    connect(m_keyboardManager, &KeyboardManager::capsLockStateChanged,
            this, &ThemeWidget::capsLockStateChangedHandle);
    connect(m_commonWidget, &CommonWidget::changeLayoutRequest,
            m_keyboardManager, &KeyboardManager::changeLayout);
    connect(m_keyboardManager, &KeyboardManager::layoutChanged,
            m_commonWidget, &CommonWidget::changeLayout);
    m_commonWidget->changeLayout(m_keyboardManager->getCurrentLayout());

    m_virtualKeyboardManager = findManager<VirtualKeyboardManager*>("virtualkeyboard");
    connect(m_commonWidget, &CommonWidget::openVirtualKeyboardRequest,
            m_virtualKeyboardManager, &VirtualKeyboardManager::open);
    connect(m_virtualKeyboardManager, &VirtualKeyboardManager::errorOccurred,
            this, &ThemeWidget::virtualKeyboardErrorHandle);

    m_shutdownManager = findManager<ShutdownManager*>("shutdown");
    connect(m_commonWidget, &CommonWidget::rebootRequest,
            this, &ThemeWidget::rebootHandle);
    connect(m_commonWidget, &CommonWidget::shutdownRequest,
            this, &ThemeWidget::shutdownHandle);

    QList<QAction*> actions;

    m_changeAuthPageAction = new QAction();
    connect(m_loginWidget, &LoginWidget::authPageChanged,
            this, &ThemeWidget::authPageChanged);
    connect(m_changeAuthPageAction, &QAction::triggered,
            this, &ThemeWidget::changeAuthPageActionTriggered);
    authPageChanged();
    actions << m_changeAuthPageAction;

    m_connectionManager = findManager<ConnectionManager*>("connection");
    const auto exitAction = m_connectionManager->exitAction();
    if (exitAction.isAvailable) {
        const auto action = new QAction(exitAction.text);
        connect(action, &QAction::triggered,
                this, &ThemeWidget::exitActionTriggered);
        actions << action;
    }

    m_consoleAuthManager = findManager<ConsoleAuthManager*>("consoleAuth");
    const auto consoleLoginAction = new QAction(tr("Console login"));
    connect(consoleLoginAction, &QAction::triggered,
            this, &ThemeWidget::consoleLoginhandle);
    actions << consoleLoginAction;

    const auto rebootAction = new QAction(tr("Reboot"));
    connect(rebootAction, &QAction::triggered,
            this, &ThemeWidget::rebootHandle);
    actions << rebootAction;

    const auto shutdownAction = new QAction(tr("Shutdown"));
    connect(shutdownAction, &QAction::triggered,
            this, &ThemeWidget::shutdownHandle);
    actions << shutdownAction;

    const auto sleepAction = new QAction(tr("Sleep"));
    connect(sleepAction, &QAction::triggered,
            this, &ThemeWidget::sleepHandle);
    actions << sleepAction;

    const auto suspendToRamAction = new QAction(tr("Hibernation"));
    connect(suspendToRamAction, &QAction::triggered,
            this, &ThemeWidget::suspendToRamHandle);
    actions << suspendToRamAction;

    m_commonWidget->setActions(actions);

    m_localAuthManager = findManager<LocalAuthManager*>("localAuth");
    connect(m_localAuthManager, &LocalAuthManager::loginFailed,
            this, &ThemeWidget::performLoginFailed);
    connect(m_localAuthManager, &LocalAuthManager::successLogin,
            this, &ThemeWidget::performSuccessLogin);
    connect(m_loginWidget, &LoginWidget::loginRequest,
            this, &ThemeWidget::performLoginRequest);
    m_loginWidget->enableHostnameSelection(false); // TODO
    connect(&m_loginFailedTimer, &QTimer::timeout,
            this, &ThemeWidget::loginFailedTimerTimeout);

    m_loginWidget->setUsers(m_usersManager->users());
    m_loginWidget->setCurrentUser(m_usersManager->defaultUsername());
}

void ThemeWidget::performLoginRequest(const AuthData& data)
{
    m_loginWidget->enableLoginWidgets(false);
    m_commonWidget->resetFocus(); // TODO

    const auto selectedSessionType = m_commonWidget->selectedSessionType();
    if (!selectedSessionType.isEmpty()) {
        m_sessionsManager->setNextSessionType(selectedSessionType);
    }

    m_localAuthManager->performLoginRequest(data);
}

void ThemeWidget::performLoginFailed()
{
    m_commonWidget->enableLoginFailedWidget();
    m_loginFailedTimer.start();
}

void ThemeWidget::performSuccessLogin()
{
    m_usersManager->saveDefaultUsername(m_loginWidget->currentUsername());
}

void ThemeWidget::loginFailedTimerTimeout()
{
    m_loginFailedTimer.stop();
    m_commonWidget->enableLoginFailedWidget(false);
    m_loginWidget->clearPassword();
    m_loginWidget->enableLoginWidgets();
    m_loginWidget->setFocusOnPassword();
}

void ThemeWidget::virtualKeyboardErrorHandle(const QString& error)
{
    QMessageBox::critical(this,
                          tr("Virtual keyboard error"),
                          error);
}

void ThemeWidget::capsLockStateChangedHandle(const bool lock)
{
    m_loginWidget->enableCapsLock(lock);
}

void ThemeWidget::rebootHandle()
{
    m_commonWidget->showActionsMenu(false);
    m_shutdownManager->doShutdown(ShutdownAction::Reboot);
}

void ThemeWidget::shutdownHandle()
{
    m_commonWidget->showActionsMenu(false);
    m_shutdownManager->doShutdown(ShutdownAction::Shutdown);
}

void ThemeWidget::sleepHandle()
{
    m_commonWidget->showActionsMenu(false);
    m_shutdownManager->doShutdown(ShutdownAction::Sleep);
}

void ThemeWidget::suspendToRamHandle()
{
    m_commonWidget->showActionsMenu(false);
    m_shutdownManager->doShutdown(ShutdownAction::SuspendToRam);
}

void ThemeWidget::consoleLoginhandle()
{
    m_commonWidget->showActionsMenu(false);
    m_consoleAuthManager->performConsoleLogin();
}

void ThemeWidget::exitActionTriggered()
{
    m_commonWidget->showActionsMenu(false);
    m_connectionManager->performExitAction();
}

void ThemeWidget::changeAuthPageActionTriggered()
{
    m_loginWidget->setCurrentAuthPage(static_cast<LoginWidget::AuthPage>(m_changeAuthPageAction->data().toInt()));
}

void ThemeWidget::authPageChanged()
{
    QString text;
    auto pageToChange = LoginWidget::AuthPage::UnknownPage;

    switch (m_loginWidget->currentAuthPage()) {
    case LoginWidget::AuthPage::ShortCutsEnterUsername:
        text = tr("Enter username");
        pageToChange = LoginWidget::AuthPage::TextEnterUsername;
        break;
    case LoginWidget::AuthPage::TextEnterUsername:
        text = tr("Choose user");
        pageToChange = LoginWidget::AuthPage::ShortCutsEnterUsername;
        break;
    default:
        break;
    }

    if (text.isEmpty()) {
        m_changeAuthPageAction->setVisible(false);
    } else {
        m_changeAuthPageAction->setVisible(true);
    }

    m_changeAuthPageAction->setText(text);
    m_changeAuthPageAction->setData(static_cast<int>(pageToChange));
}

void ThemeWidget::loadTranslations()
{
    const auto locale = m_environment.locale;

    if (locale.isNull() || locale.isEmpty()) {
        return;
    }

    const auto themeDir = QFileInfo(m_themeDescription.path()).dir();
    const auto translationsFiles = themeDir.entryInfoList(QStringList() << QString("*_%1.qm").arg(locale),
                                                          QDir::Files | QDir::Readable);
    foreach (const auto file, translationsFiles) {
        const auto translator = new QTranslator();
        if (translator->load(file.absoluteFilePath())) {
            qApp->installTranslator(translator);
        } else {
            delete translator;
        }
    }
}

void ThemeWidget::setBackgroundImage(const QString& path)
{
    m_backgroundWidget->setStyleSheet(QString(
R"raw(#%1 {
    border-image: url(%2);
    })raw").arg(m_backgroundWidget->objectName()).arg(path));
}

void ThemeWidget::resizeEvent(QResizeEvent* event)
{
    BaseThemeWidget::resizeEvent(event);
    m_backgroundWidget->setGeometry(QRect(QPoint(-20, -20), size() + QSize(40, 40))); // TODO
}

template<typename T>
T ThemeWidget::findWidget(const QString& name) const
{
    return Utils::findWidget<T>(name, this);
}

template<typename T>
T ThemeWidget::findManager(const QString& id) const
{
    const auto point = m_managers->get(id);
    if (!point) {
        throw CommonException(tr("Greeter manager '%1' not found").arg(id));
    }

    const auto manager = dynamic_cast<T>(point);
    if (!manager) {
        throw CommonException(tr("Menager '%1' convertation error").arg(id));
    }

    return manager;
}

==> fly-qdm_2.11.6.tar.xz: fly-qdm/dm_greet_modern/themewidget.cpp <==
#include "themewidget.h"

#include "exceptions.h"
#include "themewidgetcreator.h"
#include "utils.h"
#include "widgets/loginwidget.h"
#include "widgets/commonwidget.h"
#include "widgets/newpassworddialog.h"
#include "managers/virtualkeyboardmanager.h"
#include "managers/keyboardmanager.h"
#include "managers/shutdownmanager.h"
#include "managers/sessionsmanager.h"
#include "managers/connectionmanager.h"
#include "managers/consoleauthmanager.h"
#include "managers/domainmanager.h"

#include <QGridLayout>
#include <QDir>
#include <QMessageBox>
#include <QAction>
#include <QTranslator>
#include <QGraphicsBlurEffect>
#include <QSettings>


ThemeWidget::ThemeWidget(QWidget *parent) :
    BaseThemeWidget(parent),
    m_themeDescription("") // TODO
{
    m_loginFailedTimer.setInterval(2000); // TODO
}

ThemeWidget::~ThemeWidget()
{
}

void ThemeWidget::load(const ThemeDescription& themeDescription, GreeterManagers* managers, const EnvironmentData& environment)
{
    m_environment = environment;
    m_themeDescription = themeDescription;
    insertManagers(managers);
    loadTranslations();

    QSettings settings(QStringLiteral("/etc/X11/fly-dm/fly-dmrc"), QSettings::IniFormat);
    settings.beginGroup(QStringLiteral("X-*-Greeter"));

    m_sessionsManager = findManager<SessionsManager*>("sessions");

    const auto themePath = m_themeDescription.dirPath();

    m_commonWidget = new CommonWidget();
    m_commonWidget->load(ThemeWidgetCreator::create(themePath + "/common_widget.ui"));
    m_commonWidget->setAvailableSessions(m_sessionsManager->availableSessions());

    m_loginWidget = new LoginWidget();
    const auto sourceLoginWidget = ThemeWidgetCreator::create(themePath + "/central_widget.ui");
    m_loginWidget->load(sourceLoginWidget, m_themeDescription);
    m_loginWidget->enableHostnameSelection(false);

    m_backgroundWidget = new QWidget(this);
    m_backgroundWidget->setObjectName("backgroundWidget");
    const auto blurEffect = new QGraphicsBlurEffect();
    blurEffect->setBlurRadius(20.0); // TODO
    m_backgroundWidget->setGraphicsEffect(blurEffect);
    setBackgroundImage(themePath + "/background.png");

    const auto layout = new QGridLayout();
    layout->setMargin(0);
    layout->addWidget(m_commonWidget);
    setLayout(layout);

    const auto centralGridLayout = findWidget<QGridLayout*>("centralGridLayout");
    centralGridLayout->addWidget(m_loginWidget);

    const auto dateTimeWidget = findWidget<QWidget*>("dateTimeWidget");
    m_dateTimeWidget.load(dateTimeWidget);
    m_dateTimeWidget.start();

    m_usersManager = findManager<UsersManager*>("users");

    m_keyboardManager = findManager<KeyboardManager*>("keyboard");
    connect(m_keyboardManager, &KeyboardManager::capsLockStateChanged,
            this, &ThemeWidget::capsLockStateChangedHandle);
    connect(m_commonWidget, &CommonWidget::changeLayoutRequest,
            m_keyboardManager, &KeyboardManager::changeLayout);
    connect(m_keyboardManager, &KeyboardManager::layoutChanged,
            m_commonWidget, &CommonWidget::changeLayout);
    m_commonWidget->changeLayout(m_keyboardManager->getCurrentLayout());

    m_virtualKeyboardManager = findManager<VirtualKeyboardManager*>("virtualkeyboard");
    connect(m_commonWidget, &CommonWidget::openVirtualKeyboardRequest,
            m_virtualKeyboardManager, &VirtualKeyboardManager::open);
    connect(m_virtualKeyboardManager, &VirtualKeyboardManager::errorOccurred,
            this, &ThemeWidget::virtualKeyboardErrorHandle);

    m_shutdownManager = findManager<ShutdownManager*>("shutdown");
    connect(m_commonWidget, &CommonWidget::rebootRequest,
            this, &ThemeWidget::rebootHandle);
    connect(m_commonWidget, &CommonWidget::shutdownRequest,
            this, &ThemeWidget::shutdownHandle);

    QList<QAction*> actions;

    m_changeAuthPageAction = new QAction();
    connect(m_loginWidget, &LoginWidget::authPageChanged,
            this, &ThemeWidget::authPageChanged);
    connect(m_changeAuthPageAction, &QAction::triggered,
            this, &ThemeWidget::changeAuthPageActionTriggered);
    authPageChanged();

    if (m_usersManager->showUsers && (!m_usersManager->users().isEmpty() || m_usersManager->showTokenUsers)) {
        actions << m_changeAuthPageAction;
    }

    m_connectionManager = findManager<ConnectionManager*>("connection");
    const auto exitAction = m_connectionManager->exitAction();
    if (exitAction.isAvailable) {
        const auto action = new QAction(exitAction.text);
        connect(action, &QAction::triggered,
                this, &ThemeWidget::exitActionTriggered);
        actions << action;
    }

    m_consoleAuthManager = findManager<ConsoleAuthManager*>("consoleAuth");
    const auto consoleLoginAction = new QAction(tr("Console login"));
    connect(consoleLoginAction, &QAction::triggered,
            this, &ThemeWidget::consoleLoginhandle);
    actions << consoleLoginAction;

    const auto rebootAction = new QAction(tr("Reboot"));
    connect(rebootAction, &QAction::triggered,
            this, &ThemeWidget::rebootHandle);
    actions << rebootAction;

    const auto shutdownAction = new QAction(tr("Shutdown"));
    connect(shutdownAction, &QAction::triggered,
            this, &ThemeWidget::shutdownHandle);
    actions << shutdownAction;

    const auto sleepAction = new QAction(tr("Sleep"));
    connect(sleepAction, &QAction::triggered,
            this, &ThemeWidget::sleepHandle);
    actions << sleepAction;

    const auto suspendToRamAction = new QAction(tr("Hibernation"));
    connect(suspendToRamAction, &QAction::triggered,
            this, &ThemeWidget::suspendToRamHandle);
    actions << suspendToRamAction;

    m_commonWidget->setActions(actions);

    m_localAuthManager = findManager<LocalAuthManager*>("localAuth");
    connect(m_localAuthManager, &LocalAuthManager::loginFailed,
            this, &ThemeWidget::performLoginFailed);
    connect(m_localAuthManager, &LocalAuthManager::successLogin,
            this, &ThemeWidget::performSuccessLogin);
    connect(m_localAuthManager, &LocalAuthManager::passwordChangeRequested,
            this, &ThemeWidget::openNewPasswordDialog);

    DomainSettings domainSettings;
    domainSettings.domains = settings.value(QStringLiteral("Domains")).toString().split(QStringLiteral(":"), QString::SkipEmptyParts);
    domainSettings.separator = settings.value(QStringLiteral("Separator"), QVariant("@")).toString();
    domainSettings.isPrefixDomain = settings.value(QStringLiteral("IsPrefixDomain")).toBool();

    m_domainManager = new DomainManager(domainSettings, this);
    connect(m_loginWidget, &LoginWidget::loginRequest,
            m_domainManager, &DomainManager::handleLoginRequestData);
    connect(m_domainManager, &DomainManager::loginRequestDataHandled,
            this, &ThemeWidget::performLoginRequest);
    connect(m_domainManager, &DomainManager::errorOccurred,
            this, &ThemeWidget::performLoginFailed);
    connect(m_domainManager, &DomainManager::domainsListChanged,
            m_loginWidget, &LoginWidget::updateHostsList);

    connect(&m_loginFailedTimer, &QTimer::timeout,
            this, &ThemeWidget::loginFailedTimerTimeout);

    if (m_usersManager->showUsers) {
        m_loginWidget->setUsers(m_usersManager->users(), m_usersManager->showTokenUsers);
        connect(m_usersManager, &UsersManager::usersChanged, m_loginWidget, &LoginWidget::updateUsers);
        m_loginWidget->setCurrentUser(m_usersManager->defaultUsername());
    }
}

void ThemeWidget::performLoginRequest(const AuthData& data)
{
    m_loginWidget->enableLoginWidgets(false);
    m_commonWidget->resetFocus(); // TODO

    const auto selectedSessionType = m_commonWidget->selectedSessionType();
    if (!selectedSessionType.isEmpty()) {
        m_sessionsManager->setNextSessionType(selectedSessionType);
    }

    m_localAuthManager->performLoginRequest(data);
}

void ThemeWidget::performLoginFailed()
{
    m_commonWidget->enableLoginFailedWidget();
    m_loginFailedTimer.start();
}

void ThemeWidget::performSuccessLogin()
{
    m_usersManager->saveDefaultUsername(m_loginWidget->currentUsername());
}

void ThemeWidget::loginFailedTimerTimeout()
{
    m_loginFailedTimer.stop();
    m_commonWidget->enableLoginFailedWidget(false);
    m_loginWidget->clearPassword();
    m_loginWidget->enableLoginWidgets();
    m_loginWidget->setFocusOnPassword();
}

void ThemeWidget::virtualKeyboardErrorHandle(const QString& error)
{
    QMessageBox::critical(this,
                          tr("Virtual keyboard error"),
                          error);
}

void ThemeWidget::capsLockStateChangedHandle(const bool lock)
{
    m_loginWidget->enableCapsLock(lock);
}

void ThemeWidget::rebootHandle()
{
    m_commonWidget->showActionsMenu(false);
    m_shutdownManager->doShutdown(ShutdownAction::Reboot);
}

void ThemeWidget::shutdownHandle()
{
    m_commonWidget->showActionsMenu(false);
    m_shutdownManager->doShutdown(ShutdownAction::Shutdown);
}

void ThemeWidget::sleepHandle()
{
    m_commonWidget->showActionsMenu(false);
    m_shutdownManager->doShutdown(ShutdownAction::Sleep);
}

void ThemeWidget::suspendToRamHandle()
{
    m_commonWidget->showActionsMenu(false);
    m_shutdownManager->doShutdown(ShutdownAction::SuspendToRam);
}

void ThemeWidget::consoleLoginhandle()
{
    m_commonWidget->showActionsMenu(false);
    m_consoleAuthManager->performConsoleLogin();
}

void ThemeWidget::exitActionTriggered()
{
    m_commonWidget->showActionsMenu(false);
    m_connectionManager->performExitAction();
}

void ThemeWidget::changeAuthPageActionTriggered()
{
    m_loginWidget->setCurrentAuthPage(static_cast<LoginWidget::AuthPage>(m_changeAuthPageAction->data().toInt()));
}

void ThemeWidget::authPageChanged()
{
    QString text;
    auto pageToChange = LoginWidget::AuthPage::UnknownPage;

    switch (m_loginWidget->currentAuthPage()) {
    case LoginWidget::AuthPage::ShortCutsEnterUsername:
        text = tr("Enter username");
        pageToChange = LoginWidget::AuthPage::TextEnterUsername;
        break;
    case LoginWidget::AuthPage::TextEnterUsername:
        text = tr("Choose user");
        pageToChange = LoginWidget::AuthPage::ShortCutsEnterUsername;
        break;
    default:
        break;
    }

    if (text.isEmpty()) {
        m_changeAuthPageAction->setVisible(false);
    } else {
        m_changeAuthPageAction->setVisible(true);
    }

    m_changeAuthPageAction->setText(text);
    m_changeAuthPageAction->setData(static_cast<int>(pageToChange));
}

void ThemeWidget::openNewPasswordDialog(const bool withAuth)
{
    NewPasswordDialog *newPasswordDialog = new NewPasswordDialog(m_commonWidget, withAuth);

    connect(newPasswordDialog, &NewPasswordDialog::insertionCompleted,
            this, &ThemeWidget::onNewPasswordInsertionCompleted);
    connect(this, &ThemeWidget::newPasswordInsertionCompleted,
            m_localAuthManager, &LocalAuthManager::newPasswordInsertionCompleted);
    connect(newPasswordDialog, &NewPasswordDialog::rejected,
            m_localAuthManager, &LocalAuthManager::newPasswordInsertionRejected);
    connect(m_localAuthManager, &LocalAuthManager::successLogin,
            newPasswordDialog, &NewPasswordDialog::acceptDialog);
    connect(m_localAuthManager, &LocalAuthManager::pamErrorReceived,
            newPasswordDialog, &NewPasswordDialog::showErrorMessage);
    connect(newPasswordDialog, &NewPasswordDialog::rejected,
            newPasswordDialog, &NewPasswordDialog::deleteLater);

    newPasswordDialog->exec();
}

void ThemeWidget::loadTranslations()
{
    const auto locale = m_environment.locale;

    if (locale.isNull() || locale.isEmpty()) {
        return;
    }

    const auto themeDir = QFileInfo(m_themeDescription.path()).dir();
    const auto translationsFiles = themeDir.entryInfoList(QStringList() << QString("*_%1.qm").arg(locale),
                                                          QDir::Files | QDir::Readable);
    foreach (const auto file, translationsFiles) {
        const auto translator = new QTranslator();
        if (translator->load(file.absoluteFilePath())) {
            qApp->installTranslator(translator);
        } else {
            delete translator;
        }
    }
}

void ThemeWidget::setBackgroundImage(const QString& path)
{
    m_backgroundWidget->setStyleSheet(QString(
R"raw(#%1 {
    border-image: url(%2);
    })raw").arg(m_backgroundWidget->objectName()).arg(path));
}

void ThemeWidget::resizeEvent(QResizeEvent* event)
{
    BaseThemeWidget::resizeEvent(event);
    m_backgroundWidget->setGeometry(QRect(QPoint(-20, -20), size() + QSize(40, 40))); // TODO
}

void ThemeWidget::onNewPasswordInsertionCompleted(const QString &oldPassword,
                                                  const QString &newPassword,
                                                  const QString &confirmPassword)
{
    emit newPasswordInsertionCompleted(m_loginWidget->currentUsername(),
                                       oldPassword.isEmpty() ? m_loginWidget->currentPassword() : oldPassword,
                                       newPassword,
                                       confirmPassword);
}

template<typename T>
T ThemeWidget::findWidget(const QString& name) const
{
    return Utils::findWidget<T>(name, this);
}

template<typename T>
T ThemeWidget::findManager(const QString& id) const
{
    const auto point = m_managers->get(id);
    if (!point) {
        throw CommonException(tr("Greeter manager '%1' not found").arg(id));
    }

    const auto manager = dynamic_cast<T>(point);
    if (!manager) {
        throw CommonException(tr("Menager '%1' convertation error").arg(id));
    }

    return manager;
}
